package simulacion;



/**
 * Clase para representar matrices de nmeros reales.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class Matriz {    
    private double[][] datos;    
    
    /**
     * Construye la matriz con todo 1 en la diagonal de dimensiones nxm
     * @param n dimensin filas
     * @parama m dimensin columnas
     */
    public Matriz(int n, int m) {
        datos = new double[n][m];        
        for (int i = 0; i < getDimensionFila() && i < getDimensionColumna(); i++) {
            setIJ(i, i, 1);
        }
    }
    
    /**
     * Construye una matriz copia de otra dada.
     * @param m matriz a copiar.
     */
    public Matriz(Matriz m) {
        if (m != null) {
            datos = new double[m.getDimensionFila()][m.getDimensionColumna()];        
            for (int i = 0; i < getDimensionFila(); i++) {
                for (int j = 0; j < getDimensionColumna(); j++) {                    
                    setIJ(i, j, m.getIJ(i,j));
                }
            }
        }
    }
        
    /**
     * Retorna el nmero de filas de la matriz.
     * @return nmero de filas de la matriz.
     */
    public int getDimensionFila() {
        return datos.length;
    }
    
     /**
     * Retorna el nmero de columnas de la matriz.
     * @return nmero de columnas de la matriz.
     */
    public int getDimensionColumna() {
        if (datos.length > 0) {
            return datos[0].length;
        } else {
            return 0;
        }
    }
    /**
     * Retorna el elemento (i,j) de la matriz.
     * @param i fila.
     * @param j columna.
     * @return valor (i,j) de la matriz.
     */
    public double getIJ(int i, int j) {
        return datos[i][j];
    }        
    
    /**
     * Modifica la posicin (i,j) de la matriz.
     * @param i fila a cambiar.
     * @param j columna a cambiar.
     * @para valor nuevo valor.
     */
    public void setIJ(int i, int j, double valor) {
        datos[i][j] = valor;        
    }
    
    
    
    /**
     * Retorna la matriz suma de la propietaria del mtodo con el argumento.
     * @param m matriz a sumar.
     * @return matriz suma de la propietaria del mtodo y m. 
     * Si no tienen la mismas dimensiones, se retornar null
     */
    public Matriz sumar(Matriz m) {
        Matriz res = null;
        if (m.getDimensionFila() == getDimensionFila() && m.getDimensionColumna() == getDimensionColumna()) {
            res = new Matriz(m);
            for (int i = 0; i < getDimensionFila(); i++) {
                for (int j = 0; j < getDimensionColumna(); j++) {
                    res.setIJ(i, j, res.getIJ(i,j) + getIJ(i,j));
                }
            }
        }
        return res;
    }
    
    /**
     * Retorna una nueva matriz con la multiplicacin de la propietaria del mtodo 
     * y el argumento.
     * @param m matriz a multiplicar. Si la dimensin de columas de
     * la propietaria no coincide con la dimensin de filas de m, se retornar null.
     * @return matriz multiplicacin de la propietaria del mtodo y m. 
     * Si la dimensin de columas de la propietaria 
     * no coincide con la dimensin de filas de m, se retornar null.
     */
    public Matriz multiplicar(Matriz m) {
        Matriz res = null;
        if (getDimensionColumna() == m.getDimensionFila()) {
            res = new Matriz(getDimensionFila(), m.getDimensionColumna());
            for (int i = 0; i < res.getDimensionFila(); i++) {
                for (int j = 0; j < res.getDimensionColumna(); j++) {
                    res.setIJ(i, j, elementoIJMultiplicacion(i,j, m));
                }
            }
        }
        return res;
    }
    
     /**
     * Mtodo auxiliar para la multiplicacion de matrices.
     * Halla el elemento (fil,col) de la multiplicacin
     * de la matriz propietaria y m.
     * @param fil fila a hallar multiplicacin
     * @param col columna a hallar multiplicacin
     * @param m matriz a multiplicar con propietaria
     * @return elemento (fil, col) de la multiplicacin de matriz 
     * propietaria y m
     */    
    private double elementoIJMultiplicacion(int fil, int col, Matriz m) {
        double sum = 0;
        for (int i = 0; i < getDimensionColumna(); i++) {
            sum += getIJ(fil, i) * m.getIJ(i, col);
        }
        return sum;
    }    
    
    /**
     * Retorna un String con la informacin de la matriz. 
     * @return String, en cada lnea del String se encuentra 
     * la informacin de una fila.
     */
    public String toString() {
        String res = "";
        for (int i = 0; i < getDimensionFila(); i++) {
            for (int j = 0; j < getDimensionColumna(); j++) {
                res += getIJ(i, j) + " ";
            }
            res += "\n";
        }
        return res;
    }
}